==> chainwallet-1.0.2.tar.gz: lib/src/core/coretypes/Currency.dart <==
import 'dart:core';
import "dart:typed_data";

import "../fields/Type.dart";
import "../serialized/BytesSink.dart";


import 'package:chainwallet/src/utils/Util.dart';


import 'package:chainwallet/src/core/coretypes/hash/Hash160.dart';


import 'package:chainwallet/src/core/coretypes/hash/Hash.dart';
/**
 * Originally it was intended that AccountIDs would be variable length so that's
 * why they are variable length encoded as top level field objects.
 *
 * Note however, that in practice, all account ids are just 160 bit hashes.
 * Consider the fields TakerPaysIssuer and fixed length encoding of issuers in
 * amount serializations.
 *
 * Thus, we extend Hash160 which affords us some functionality.
 */
class Currency extends Hash160 {

  static CurrencyTranslator translate = new CurrencyTranslator();

  static  Currency ZXC = new Currency(Util.createUint8ListFromBigInt(BigInt.zero));

  static Currency fromString(String currency) {
    return translate.fromString(currency);
  }


  static Uint8List encodeCurrency(String currencyCode) {

    Uint8List currencyBytes = new Uint8List(20);
    Uint8List currency = Util.createUint8ListFromString(currencyCode);

    if(currency.length < 3)
      return currencyBytes;

    currencyBytes[12] = currency[0];
    currencyBytes[13] = currency[1];
    currencyBytes[14] = currency[2];
    return currencyBytes;
  }


  Currency(Uint8List bytes):super(bytes) {

  }




  Map toJSON() => null;
  Uint8List toBytes() => translate.toBytes(this);
  String toHex() => null;


  void toBytesSink(BytesSink to) => translate.toBytesSink(this, to);

  Type type() => Type.AccountID;

  static String isoCodeFromBytesAndOffset(Uint8List bytes, int offset) {

      Uint8List sub = bytes.sublist(offset,offset+3);
      return new String.fromCharCodes(sub);;
  }


  static String getCurrencyCodeFromTLCBytes(Uint8List bytes) {
        int i;
        bool zeroInNonCurrencyBytes = true;

        for (i = 0; i < 20; i++) {
            zeroInNonCurrencyBytes = zeroInNonCurrencyBytes &&
            ((i == 12 || i == 13 || i == 14) || // currency bytes (0 or any other)
            bytes[i] == 0);                   // non currency bytes (0)
        }

        if (zeroInNonCurrencyBytes) {
           return isoCodeFromBytesAndOffset(bytes, 12);
        } else {
            throw new Exception("Currency is invalid");
        }
  }


  String toString() {
//    switch (type) {
//      case ISO:
    String code = getCurrencyCodeFromTLCBytes(bytes());
    if (code ==("ZXC")) {
      // HEX of the bytes
      return super.toString();
    } else if (code == ("\0\0\0")) {
      return "ZXC";
    } else {
      // the 3 letter isoCode
      return code;
    }
//      case HASH:
//      case DEMURRAGE:
//      case UNKNOWN:
//      default:
//        return super.toString();
  }



}





class  CurrencyTranslator extends   HashTranslator<Currency> {


  Currency newInstance(Uint8List b) {
         return new Currency(b);
  }

  int byteWidth() =>20;

//  AccountID fromParser(BinaryParser parser, int hint) {
//    return newInstance(parser.read(byteWidth()));
//  }


  Currency fromString(String value) {
    if (value.length == 40 /* byteWidth() * 2 */) {

     // TODO
      return newInstance(Util.createUint8ListFromString(value));
    } else if (value == ("ZXC")) {
      return Currency.ZXC;
    } else {
      if (!(value.length == 3)) {
//                if (!value.matches("[A-Z0-9]{3}")) {
        throw new Exception("Currency code must be 3 characters");
      }
      return newInstance(Currency.encodeCurrency(value));
    }
  }



}

==> chainwallet-1.0.4.tar.gz: lib/src/core/coretypes/Currency.dart <==
import 'dart:core';
import "dart:typed_data";

import "../fields/Type.dart";
import "../serialized/BytesSink.dart";

import 'package:chainwallet/src/utils/Util.dart';

import 'package:chainwallet/src/core/coretypes/hash/Hash160.dart';

import 'package:chainwallet/src/core/coretypes/hash/Hash.dart';

/**
 * Originally it was intended that AccountIDs would be variable length so that's
 * why they are variable length encoded as top level field objects.
 *
 * Note however, that in practice, all account ids are just 160 bit hashes.
 * Consider the fields TakerPaysIssuer and fixed length encoding of issuers in
 * amount serializations.
 *
 * Thus, we extend Hash160 which affords us some functionality.
 */
class Currency extends Hash160 {
  static CurrencyTranslator translate = new CurrencyTranslator();

  static Currency ZXC =
      new Currency(Util.createUint8ListFromBigInt(BigInt.zero));

  static Currency fromString(String currency) {
    return translate.fromString(currency);
  }

  static Uint8List encodeCurrency(String currencyCode) {
    Uint8List currencyBytes = new Uint8List(20);
    Uint8List currency = Util.createUint8ListFromString(currencyCode);

    if (currency.length < 3) return currencyBytes;

    currencyBytes[12] = currency[0];
    currencyBytes[13] = currency[1];
    currencyBytes[14] = currency[2];
    return currencyBytes;
  }

  Currency(Uint8List bytes) : super(bytes) {}

  Map toJSON() => null;
  Uint8List toBytes() => translate.toBytes(this);
  String toHex() => null;

  void toBytesSink(BytesSink to) => translate.toBytesSink(this, to);

  Type type() => Type.AccountID;

  static String isoCodeFromBytesAndOffset(Uint8List bytes, int offset) {
    Uint8List sub = bytes.sublist(offset, offset + 3);
    return new String.fromCharCodes(sub);
    ;
  }

  static String getCurrencyCodeFromTLCBytes(Uint8List bytes) {
    int i;
    bool zeroInNonCurrencyBytes = true;

    for (i = 0; i < 20; i++) {
      zeroInNonCurrencyBytes = zeroInNonCurrencyBytes &&
          ((i == 12 || i == 13 || i == 14) || // currency bytes (0 or any other)
              bytes[i] == 0); // non currency bytes (0)
    }

    if (zeroInNonCurrencyBytes) {
      return isoCodeFromBytesAndOffset(bytes, 12);
    } else {
      throw new Exception("Currency is invalid");
    }
  }

  String toString() {
//    switch (type) {
//      case ISO:
    String code = getCurrencyCodeFromTLCBytes(bytes());
    if (code == ("ZXC")) {
      // HEX of the bytes
      return super.toString();
    } else if (code == ("\0\0\0")) {
      return "ZXC";
    } else {
      // the 3 letter isoCode
      return code;
    }
//      case HASH:
//      case DEMURRAGE:
//      case UNKNOWN:
//      default:
//        return super.toString();
  }
}

class CurrencyTranslator extends HashTranslator<Currency> {
  Currency newInstance(Uint8List b) {
    return new Currency(b);
  }

  int byteWidth() => 20;

//  AccountID fromParser(BinaryParser parser, int hint) {
//    return newInstance(parser.read(byteWidth()));
//  }

  Currency fromString(String value) {
    if (value.length == 40 /* byteWidth() * 2 */) {
      // TODO
      return newInstance(Util.createUint8ListFromString(value));
    } else if (value == ("ZXC")) {
      return Currency.ZXC;
    } else {
      if (!(value.length == 3)) {
//                if (!value.matches("[A-Z0-9]{3}")) {
        throw new Exception("Currency code must be 3 characters");
      }
      return newInstance(Currency.encodeCurrency(value));
    }
  }
}
